'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reactIntl = require('react-intl');

exports.default = configContext => {
  var _configContext$inputC = configContext.inputComponents;
  const CompoundInput = _configContext$inputC.CompoundInput,
        DateInput = _configContext$inputC.DateInput,
        OptionPickerInput = _configContext$inputC.OptionPickerInput,
        TextInput = _configContext$inputC.TextInput,
        AutocompleteInput = _configContext$inputC.AutocompleteInput,
        IDGeneratorInput = _configContext$inputC.IDGeneratorInput,
        TermPickerInput = _configContext$inputC.TermPickerInput;
  const config = configContext.configHelpers.configKey;
  const DATA_TYPE_DATE = configContext.dataTypes.DATA_TYPE_DATE;
  const extensions = configContext.config.extensions;


  return {
    document: Object.assign({
      [config]: {
        view: {
          type: CompoundInput,
          props: {
            defaultChildSubpath: 'ns2:loansout_common'
          }
        }
      }
    }, extensions.core.fields, {
      'ns2:loansout_common': {
        [config]: {
          service: {
            ns: 'http://collectionspace.org/services/loanout'
          }
        },
        loanOutNumber: {
          [config]: {
            cloneable: false,
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.loanOutNumber.name',
                'defaultMessage': 'Loan out number'
              }
            }),
            required: true,
            searchView: {
              type: TextInput
            },
            view: {
              type: IDGeneratorInput,
              props: {
                source: 'loanout'
              }
            }
          }
        },
        loanPurpose: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.loanPurpose.name',
                'defaultMessage': 'Loan purpose'
              }
            }),
            view: {
              type: OptionPickerInput,
              props: {
                source: 'loanPurposes'
              }
            }
          }
        },
        lendersAuthorizer: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.lendersAuthorizer.name',
                'defaultMessage': 'Authorizer'
              },
              fullName: {
                'id': 'field.loansout_common.lendersAuthorizer.fullName',
                'defaultMessage': 'Lender authorizer'
              }
            }),
            view: {
              type: AutocompleteInput,
              props: {
                source: 'person/local,person/shared'
              }
            }
          }
        },
        lendersContact: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.lendersContact.name',
                'defaultMessage': 'Contact'
              },
              fullName: {
                'id': 'field.loansout_common.lendersContact.fullName',
                'defaultMessage': 'Lender contact'
              }
            }),
            view: {
              type: AutocompleteInput,
              props: {
                source: 'person/local,person/shared'
              }
            }
          }
        },
        lendersAuthorizationDate: {
          [config]: {
            dataType: DATA_TYPE_DATE,
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.lendersAuthorizationDate.name',
                'defaultMessage': 'Authorization date'
              },
              fullName: {
                'id': 'field.loansout_common.lendersAuthorizationDate.fullName',
                'defaultMessage': 'Lender authorization date'
              }
            }),
            view: {
              type: DateInput
            }
          }
        },
        borrower: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.borrower.name',
                'defaultMessage': 'Name'
              },
              fullName: {
                'id': 'field.loansout_common.borrower.fullName',
                'defaultMessage': 'Borrower name'
              }
            }),
            view: {
              type: AutocompleteInput,
              props: {
                source: 'person/local,person/shared,organization/local,organization/shared'
              }
            }
          }
        },
        borrowersContact: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.borrowersContact.name',
                'defaultMessage': 'Contact'
              },
              fullName: {
                'id': 'field.loansout_common.borrowersContact.fullName',
                'defaultMessage': 'Borrower contact'
              }
            }),
            view: {
              type: AutocompleteInput,
              props: {
                source: 'person/local,person/shared'
              }
            }
          }
        },
        borrowersAuthorizer: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.borrowersAuthorizer.name',
                'defaultMessage': 'Authorizer'
              },
              fullName: {
                'id': 'field.loansout_common.borrowersAuthorizer.fullName',
                'defaultMessage': 'Borrower authorizer'
              }
            }),
            view: {
              type: AutocompleteInput,
              props: {
                source: 'person/local,person/shared'
              }
            }
          }
        },
        borrowersAuthorizationDate: {
          [config]: {
            dataType: DATA_TYPE_DATE,
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.borrowersAuthorizationDate.name',
                'defaultMessage': 'Authorization date'
              },
              fullName: {
                'id': 'field.loansout_common.borrowersAuthorizationDate.fullName',
                'defaultMessage': 'Borrower authorization date'
              }
            }),
            view: {
              type: DateInput
            }
          }
        },
        specialConditionsOfLoan: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.specialConditionsOfLoan.name',
                'defaultMessage': 'Conditions of loan'
              }
            }),
            view: {
              type: TextInput,
              props: {
                multiline: true
              }
            }
          }
        },
        loanOutNote: {
          [config]: {
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.loanOutNote.name',
                'defaultMessage': 'Note'
              }
            }),
            view: {
              type: TextInput,
              props: {
                multiline: true
              }
            }
          }
        },
        loanStatusGroupList: {
          [config]: {
            view: {
              type: CompoundInput
            }
          },
          loanStatusGroup: {
            [config]: {
              messages: (0, _reactIntl.defineMessages)({
                name: {
                  'id': 'field.loansout_common.loanStatusGroup.name',
                  'defaultMessage': 'Loan status'
                }
              }),
              repeating: true,
              view: {
                type: CompoundInput,
                props: {
                  tabular: true
                }
              }
            },
            loanStatus: {
              [config]: {
                messages: (0, _reactIntl.defineMessages)({
                  name: {
                    'id': 'field.loansout_common.loanStatus.name',
                    'defaultMessage': 'Status'
                  },
                  fullName: {
                    'id': 'field.loansout_common.loanStatus.fullName',
                    'defaultMessage': 'Loan status'
                  }
                }),
                view: {
                  type: TermPickerInput,
                  props: {
                    source: 'loanoutstatus'
                  }
                }
              }
            },
            loanStatusDate: {
              [config]: {
                dataType: DATA_TYPE_DATE,
                messages: (0, _reactIntl.defineMessages)({
                  name: {
                    'id': 'field.loansout_common.loanStatusDate.name',
                    'defaultMessage': 'Date'
                  },
                  fullName: {
                    'id': 'field.loansout_common.loanStatusDate.fullName',
                    'defaultMessage': 'Loan status date'
                  }
                }),
                view: {
                  type: DateInput
                }
              }
            },
            loanStatusNote: {
              [config]: {
                messages: (0, _reactIntl.defineMessages)({
                  name: {
                    'id': 'field.loansout_common.loanStatusNote.name',
                    'defaultMessage': 'Note'
                  }
                }),
                view: {
                  type: TextInput
                }
              }
            }
          }
        },
        loanOutDate: {
          [config]: {
            dataType: DATA_TYPE_DATE,
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.loanOutDate.name',
                'defaultMessage': 'Loan out date'
              }
            }),
            view: {
              type: DateInput
            }
          }
        },
        loanReturnDate: {
          [config]: {
            dataType: DATA_TYPE_DATE,
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.loanReturnDate.name',
                'defaultMessage': 'Loan return date'
              }
            }),
            view: {
              type: DateInput
            }
          }
        },
        loanRenewalApplicationDate: {
          [config]: {
            dataType: DATA_TYPE_DATE,
            messages: (0, _reactIntl.defineMessages)({
              name: {
                'id': 'field.loansout_common.loanRenewalApplicationDate.name',
                'defaultMessage': 'Loan renewal application date'
              }
            }),
            view: {
              type: DateInput
            }
          }
        }
      }
    })
  };
};